from acmin.models import Permission, ContentType
from acmin.utils import attr


class AccessMixin:
    from django.contrib.auth import REDIRECT_FIELD_NAME
    login_url = None
    permission_denied_message = ''
    raise_exception = False
    redirect_field_name = REDIRECT_FIELD_NAME

    def get_login_url(self):
        from django.core.exceptions import ImproperlyConfigured
        from django.conf import settings
        login_url = self.login_url or settings.LOGIN_URL
        if not login_url:
            raise ImproperlyConfigured(
                '{0} is missing the login_url attribute. Define {0}.login_url, settings.LOGIN_URL, or override '
                '{0}.get_login_url().'.format(self.__class__.__name__)
            )
        return str(login_url)

    def get_permission_denied_message(self):
        return self.permission_denied_message

    def get_redirect_field_name(self):
        return self.redirect_field_name

    def handle_no_permission(self):
        from django.contrib.auth.views import redirect_to_login
        from django.core.exceptions import PermissionDenied
        if self.raise_exception or self.request.user.is_authenticated:
            raise PermissionDenied(self.get_permission_denied_message())
        return redirect_to_login(self.request.get_full_path(), self.get_login_url(), self.get_redirect_field_name())

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated and self.has_permission():
            return super().dispatch(request, *args, **kwargs)
        else:
            return self.handle_no_permission()

    def has_permission(self):
        return True


class ContextMixin(object):
    @property
    def user(self):
        return attr(self, "request.user")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs) or {}
        model = self.model
        model.class_permission = Permission.get_permission(self.user, model)
        context.update({
            'model': self.model,
            'model_name': self.model.__name__,
            'model_verbose_name': attr(ContentType.get_by_user_and_model(self.user, model), "verbose_name"),
        })
        return context



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        from acmin.models import Group, User, GroupPermission, SuperPermissionModel, ContentType
        group = Group.objects.update_or_create(name="admin")[0]

        user = User.objects.filter(username="admin").first()
        if not user:
            user = User(group=group, username="admin", title="admin")
            user.set_password("123456")
            user.save()

        contenttype = ContentType.objects.filter(name=SuperPermissionModel.__name__).first()
        permission = GroupPermission.objects.filter(group=group, contenttype=contenttype).first()
        if not permission:
            GroupPermission.objects.create(
                group=group,
                contenttype=contenttype,
                name="admin",
                creatable=True,
                savable=True,
                removable=True,
                cloneable=True,
                exportable=True,
                viewable=True,
                listable=True,
            )



import collections

from django.db import models
from django.db.models.signals import post_save, post_delete
from django.dispatch import receiver
from filelock import FileLock
from python_utils import converters

from .base import AcminModel
from .group import Group
from .user import User

locker = FileLock("config.lock")
locker.release(force=True)
cache = collections.defaultdict(dict)


@receiver(post_save)
@receiver(post_delete)
def handle_model_change(sender, **kwargs):
    if sender in [Group, User, Config, UserConfig, GroupConfig]:
        with locker:
            cache.clear()


def get_configs():
    if not cache:
        with locker:
            for user in User.objects.all():
                for config in Config.objects.all():
                    cache[user][config.name] = config.value
            for config in GroupConfig.objects.all():
                for user in User.objects.filter(group=config.group).all():
                    cache[user][config.name] = config.value

            for config in UserConfig.objects.all():
                cache[user][config.name] = config.value
    return cache


class BaseConfig(AcminModel):
    class Meta:
        abstract = True

    name = models.CharField('名称', max_length=50, unique=True)
    value = models.TextField("值")

    def __str__(self):
        return self.name

    @classmethod
    def get(cls, user, name, default=None):
        return get_configs()[user].get(name, default)

    @classmethod
    def get_int(cls, user, name, default=0):
        return converters.to_int(cls.get(user, name), default)

    @classmethod
    def get_bool(cls, user, name):
        return cls.get(user, name, "").upper() in ["TRUE", "1", "YES", "Y"]


class Config(BaseConfig):
    class Meta:
        verbose_name = verbose_name_plural = '配置'


class GroupConfig(BaseConfig):
    class Meta:
        verbose_name = verbose_name_plural = '配置(用户组)'

    group = models.ForeignKey(Group, on_delete=models.CASCADE)


class UserConfig(BaseConfig):
    class Meta:
        verbose_name = verbose_name_plural = '配置(用户)'

    user = models.ForeignKey(User, on_delete=models.CASCADE)



import logging

from acmin.utils import memorize, import_class

logger = logging.getLogger(__name__)


@memorize
def get_base_view_class(app_name, action):
    try:
        return import_class(f"{app_name}.views.Base{action.capitalize()}View")
    except:
        return import_class(f"acmin.views.Admin{action.capitalize()}View")


def get_view(model, action):
    app_name = model.__module__.split(".")[0]

    view_name = "%s%sView" % (model.__name__, action.capitalize())
    view = None
    try:
        view = import_class(f'{app_name}.views.{view_name}')
    except:
        try:
            view = type("Dynamic%s" % view_name, (get_base_view_class(app_name, action),), dict(
                model=model,
                __module__=f'{app_name}.{__name__}',

            ))
        except Exception as e:
            logger.error(e)

    return view.as_view()



import logging

from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from acmin.models import Permission, PermissionItem
from acmin.serializer import get_serializer
from acmin.utils import import_class, attr, param

logger = logging.getLogger(__name__)


class BaseViewSet(viewsets.ModelViewSet):
    class Meta:
        model = None

    def get_serializer_class(self):
        result = get_serializer(self.Meta.model)
        return result

    def get_serializer_context(self):
        context = super().get_serializer_context()
        return context

    def get_serializer(self, *args, **kwargs):
        serializer = super().get_serializer(*args, **kwargs)
        return serializer

    def get_queryset(self):
        queryset = attr(self, "Meta.model").objects.all()
        sorter = param(self.request, "sorter")
        if sorter:
            queryset = queryset.order_by(sorter)
        return queryset

    def get_paginated_response(self, data):
        paginator = self.paginator.page.paginator
        count = paginator.count
        return Response(dict(
            data=dict(
                list=data,
                total=count,
                pageSize=paginator.per_page,
                pages=paginator.num_pages,
                current=int(self.request.GET.get("page", 1))
            ),
            status=200,
            message="success"
        ))


class ViewPermission(BasePermission):
    def has_permission(self, request, view):
        return Permission.has_permission(request.user, view.Meta.model, PermissionItem.listable)


class AuthenticatedBaseViewSet(BaseViewSet):
    permission_classes = (IsAuthenticated, ViewPermission,)


def get_viewset(model_class, login_required=True):
    app_name = model_class.__module__.split(".")[0]
    name = f"{model_class.__name__}ViewSet"
    module = f'{app_name}.views'
    try:
        return import_class(f'{module}.{name}')
    except(ImportError, AttributeError, Exception):
        super_class = AuthenticatedBaseViewSet if login_required else BaseViewSet
        return type(f"Dynamic{name}", (super_class,), dict(
            Meta=type("Meta", (), dict(
                model=model_class
            )),
            __module__=module,
        ))



from django.conf import settings


def extra_context(request):
    return {
        'app_name': getattr(settings, "ACMIN_APP_NAME"),
        "function_name": getattr(settings, "ACMIN_FUNCTION_NAME"),
        "url_prefix": f"/{getattr(settings,'ACMIN_ADMIN_PREFIX')}/{getattr(settings,'ACMIN_APP_NAME')}"
    }
